@@ -1,4 +1,4 @@
-# $Id: FileHandles.pm 2087 2006-09-01 10:24:43Z bsmith $
+# $Id: FileHandles.pm 2455 2009-02-22 07:45:28Z rcaputo $
 
 # Manage file handles, associated descriptors, and read/write modes
 # thereon.
@@ -6,7 +6,7 @@
 package POE::Resource::FileHandles;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2087 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2455 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -19,9 +19,14 @@ use strict;
 # aren't used if we're RUNNING_IN_HELL, but Perl needs to see them.
 
 BEGIN {
-  if (RUNNING_IN_HELL) {
-    eval '*F_GETFL = sub { 0 };';
-    eval '*F_SETFL = sub { 0 };';
+  if ( ! defined &F_GETFL ) {
+    if ( ! defined prototype "F_GETFL" ) {
+      *F_GETFL = sub { 0 };
+      *F_SETFL = sub { 0 };
+    } else {
+      *F_GETFL = sub () { 0 };
+      *F_SETFL = sub () { 0 };
+    }
   }
 }
 
@@ -35,6 +40,7 @@ my $kr_queue;
 ### more unique.
 
 my %kr_filenos;
+BEGIN { $poe_kernel->[KR_FILENOS] = \%kr_filenos; }
 
 sub FNO_MODE_RD      () { MODE_RD } # [ [ (fileno read mode structure)
 # --- BEGIN SUB STRUCT 1 ---        #
@@ -87,11 +93,6 @@ sub SH_MODECOUNT  () {  2 } #          [ $read_reference_count,     (MODE_RD)
                             #      },
                             #    },
 
-sub _data_handle_preload {
-    $poe_kernel->[KR_FILENOS] = \%kr_filenos;
-}
-use POE::API::ResLoader \&_data_handle_preload;
-
 ### Begin-run initialization.
 
 sub _data_handle_initialize {
@@ -514,7 +515,7 @@ sub _data_handle_remove {
 
   # Make sure the handle is deregistered with the kernel.
 
-  if (exists $kr_filenos{$fd}) {
+  if (defined($fd) and exists($kr_filenos{$fd})) {
     my $kr_fileno  = $kr_filenos{$fd};
     my $kr_fno_rec = $kr_fileno->[$mode];
 
@@ -526,7 +527,7 @@ sub _data_handle_remove {
     ) {
 
       TRACE_FILES and
-        _warn "<fh> removing handle ($handle) fileno ($fd) mode ($mode)";
+        _warn "<fh> removing handle ($handle) fileno ($fd) mode ($mode) from " . Carp::shortmess;
 
       # Remove the handle from the kernel's session record.
 
@@ -553,7 +554,7 @@ sub _data_handle_remove {
         $self->_data_ev_refcount_dec( @$event[EV_SESSION, EV_SOURCE] );
 
         TRACE_EVENTS and
-          _warn "<ev> removing select event $id ``$event->[EV_NAME]''";
+          _warn "<ev> removing select event $id ``$event->[EV_NAME]''" . Carp::shortmess;
 
         $kr_fno_rec->[FMO_EV_COUNT]--;
 
@@ -625,7 +626,7 @@ sub _data_handle_remove {
   }
 
   # SS_HANDLES - Remove the select from the session, assuming there is
-  # a session to remove it from.  -><- Key it on fileno?
+  # a session to remove it from.  TODO Key it on fileno?
 
   if (
     exists($kr_ses_to_handle{$session}) and
@@ -753,7 +754,7 @@ sub _data_handle_clear_session {
   }
 }
 
-# -><- Testing accessors.  Maybe useful for introspection.  May need
+# TODO Testing accessors.  Maybe useful for introspection.  May need
 # modification before that.
 
 sub _data_handle_fno_refcounts {
@@ -808,30 +809,41 @@ __END__
 
 =head1 NAME
 
-POE::Resource::FileHandles - manage file handles on behalf of POE::Kernel
+POE::Resource::FileHandles - internal filehandle manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module encapsulates low-level file handle management for
-POE::Kernel.  It provides accessors to its data structures that
-POE::Kernel uses internally.  This module has no public interface.
-Move along.
+POE::Resource::FileHandles is a mix-in class for POE::Kernel.  It
+provides the low-level features to manage filehandles.  It is used
+internally by POE::Kernel, so it has no public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> for documentation on file handles and selects.
+See L<POE::Kernel/I/O Watchers (Selects)> for the public file watcher
+API.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-Probably.
+POE watches I/O based on filehandles rather than file descriptors,
+which means there can be clashes between its API and an underlying
+descriptor-based event loop.  This is usually not a problem, but it
+may require work-arounds in certain edge cases.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
